if ("undefined" == typeof (FoxyDealInstaller) && "undefined" == typeof (FoxyDeal)) {
    var FoxyDeal = {
        initialize: (function () {
            var started = false;
            var initialized = false;
            return function () {
                /*
                * initializes FoxyDeal;
                * downloads/loads UID and domain array
                * then registers 'onNavigate' listener
                */

                // don't do anything if storage wasn't properly initialized
                if (!FoxyDeal.extension.storage) {
                    return;
                }

                // register 'navigateListener' if FoxyDeal has already been initialized
                if (initialized) {
                    FoxyDeal.extension.navigateListener.register();
                    return;
                }

                // stop further execution if initialization has already been started
                if (started) {
                    return;
                }

                // set 'started' to 'true' so that any future call doesn't cause re-initialization
                started = true;

                // load UID
                FoxyDeal.uid.load(function () {
                    // load domain array
                    FoxyDeal.rules.domains.load(function () {
                        // load rules container
                        FoxyDeal.rules.shopRules.load(function () {
                            // register 'onNavigate' listener
                            FoxyDeal.extension.navigateListener.register();
                            // set 'initalized' to 'true' so that any future call only registeres 'navigateListener'
                            initialized = true;
                        });
                    });
                });
            };
        })(),
        api: (function () {
            var protocol = 'https';
            var domain = 'api.foxydeal.com';
            var version = 'v2.0';
            return {
                baseURL: protocol + '://' + domain + '/' + version + '/',
                version: version
            };
        })(),
        cdn: (function () {
            var protocol = 'https';
            var domain = 's3.amazonaws.com';
            var path = 'cdn.foxydeal.com';
            var version = 'v6';
            return {
                baseURL: protocol + '://' + domain + '/' + path + '/' + version + '/',
                version: version
            };
        })(),
        uid: {
            value: '',
            load: function (onLoaded) {
                /*
                * loads UID from storage if it was previously stored
                * else, downloads UID from API
                * @onLoaded function which needs to be called when UID has been loaded
                */

                FoxyDeal.storage.get({
                    uid: null,
                    uidRequestedOn: ''
                }, function (storedObj) {
                    // if UID has been stored previously return it and prevent further execution
                    if (storedObj.uid) {
                        FoxyDeal.extension.log.info('uid (' + storedObj.uid + ') found in storage.');
                        FoxyDeal.uid.value = storedObj.uid;
                        onLoaded();
                        return;
                    }

                    // if UID hasn't been stored but request was sent today execute callback and prevent further execution
                    if (storedObj.uidRequestedOn === FoxyDeal.rules.time.string()) {
                        FoxyDeal.extension.log.info('uid not found in storage. Request was already sent today, new request won\'t be sent until tomorrow.');
                        onLoaded();
                        return;
                    }

                    // build URL
                    var url = FoxyDeal.api.baseURL + 'uid';
                    if (FoxyDeal.partner.id) {
                        url += '/' + FoxyDeal.partner.id;
                    }
                    if (FoxyDeal.partner.sub_id) {
                        url += '/' + FoxyDeal.partner.sub_id;
                    }
                    var postObj = {
                        version: FoxyDeal.extension.version
                    };
                    if (FoxyDeal.partner.transaction_id) {
                        postObj.transactionId = FoxyDeal.partner.transaction_id;
                    }

                    FoxyDeal.storage.set({ uidRequestedOn: FoxyDeal.rules.time.string() });

                    FoxyDeal.extension.log.info('Sending uid request, url: ' + url);

                    // download UID
                    FoxyDeal.extension.ajax(url, function (xhr) {
                        try {
                            if (xhr.status === 200) {
                                FoxyDeal.uid.value = JSON.parse(xhr.responseText).uid;
                                FoxyDeal.storage.set({ uid: FoxyDeal.uid.value });
                                FoxyDeal.extension.log.info('Got valid UID response from server: ' + xhr.responseText);
                            }
                        } catch (ex) {
                            FoxyDeal.extension.log.error('UID response was received but exception occurred while attempting to parse data.\nReceived data:\n' + (xhr.responseText ? xhr.responseText : '') + '\nException info:' + ex.message);
                        }
                        // execute callback
                        onLoaded();
                    }, {
                        postJSobj: postObj
                    });
                });
            }
        },
        partner: {
            id: 1001,
            sub_id: 1,
            transaction_id: ''
        },
        onNavigate: function (hostname, location, ext_obj) {
            /* checks if website is one of supported shops and if so, initiates rules request
            * @hostname hostname of the page which fired "onNavigate" event
            * @ext_obj extension specific object, window/tab/tab ID/etc which fired event
            */

            if (FoxyDeal.isEnabled === false) {
                return;
            }

            // prevent further execution if hostname and/or ext_obj were not passed
            if (!hostname || !ext_obj) {
                return;
            }

            // get domain if it's one of supported domains
            var supportedDomain = FoxyDeal.getDomain(hostname);

            // prevent further execution if domain is not supported
            if (!supportedDomain) {
                FoxyDeal.extension.log.info('Domain (' + hostname + ') not found in domain array.');
                return;
            }

            FoxyDeal.extension.log.info('Supported shop detected: ' + supportedDomain);

            // get rules and inject them and main.js to page
            FoxyDeal.rules.shopRules.get(supportedDomain, function (shopRules) {
                FoxyDeal.extension.log.info(JSON.stringify(shopRules));
                if (!shopRules.l) {
                    return;
                }

                var rx = new RegExp(shopRules.l);
                if (!rx.test(location)) {
                    FoxyDeal.extension.log.info('RegEx doesn\'t match - not a product page or RegEx needs to be updated.');
                    return;
                }

                // add domain to shop rules
                shopRules.d = supportedDomain;

                // create object with info which needs to be passed to main.js
                var info = {
                    uid: FoxyDeal.uid.value,
                    partner: FoxyDeal.partner,
                    rules: shopRules
                };

                // inject JS
                FoxyDeal.extension.injectJS(ext_obj, info);
            });
        },
        storage: (function () {
            var undeletable = {};
            return {
                get: function (keys, onLoaded) {
                    /*
                    * gets stored JS object(s)
                    * @keys - object which contains keys and default values
                    * @onLoaded - callback which accepts object with stored data as parameter
                    */

                    // load stored data
                    var storedData = {};
                    try {
                        storedData = JSON.parse(FoxyDeal.extension.storage.getStoredData());
                    } catch (ex) {

                    }

                    if (!storedData) {
                        storedData = {};
                    }

                    if (!undeletable.uid && storedData.uid) {
                        undeletable.uid = storedData.uid;
                    } else if (undeletable.uid && !storedData.uid) {
                        FoxyDeal.storage.set({ uid: undeletable.uid });
                    }

                    if (!undeletable.domainsContainer && storedData.domainsContainer) {
                        undeletable.domainsContainer = storedData.domainsContainer;
                    } else if (undeletable.domainsContainer && !storedData.domainsContainer) {
                        FoxyDeal.storage.set({ domainsContainer: undeletable.domainsContainer });
                    }

                    // get stored values for keys which were passed
                    var storedObject = {};
                    var key;
                    for (key in keys) {
                        if (storedData[key] === undefined) {
                            storedObject[key] = keys[key];
                        } else {
                            storedObject[key] = storedData[key];
                        }
                    }

                    // execute callback
                    onLoaded(storedObject);
                },
                set: function (obj, onSaved) {
                    /*
                    * stores JS object(s)
                    * @obj - object which needs to be stored
                    * @onSaved - optional callback which accepts object with stored data as parameter
                    */

                    // load stored data
                    var storedData = {};
                    try {
                        storedData = JSON.parse(FoxyDeal.extension.storage.getStoredData());
                    } catch (ex) {

                    }

                    if (!storedData) {
                        storedData = {};
                    }

                    // add object to other stored objects and save the collection
                    try {
                        var key;
                        for (key in obj) {
                            storedData[key] = obj[key];
                        }

                        if (!undeletable.uid && storedData.uid) {
                            undeletable.uid = storedData.uid;
                        } else if (undeletable.uid && !storedData.uid) {
                            storedData.uid = undeletable.uid;
                        }

                        if (!undeletable.domainsContainer && storedData.domainsContainer) {
                            undeletable.domainsContainer = storedData.domainsContainer;
                        } else if (undeletable.domainsContainer && !storedData.domainsContainer) {
                            storedData.domainsContainer = undeletable.domainsContainer;
                        }

                        FoxyDeal.extension.storage.setStoredData(JSON.stringify(storedData));

                        if (onSaved) {
                            // execute callback
                            onSaved(obj);
                        }
                    } catch (ex) {
                        if (onSaved) {
                            // execute callback
                            onSaved(obj);
                        }
                    }
                },
                remove: function (keysArray, onRemoved) {
                    /*
                    * removes items from storage
                    * @keysArray - array with keys which need to be deleted
                    */

                    try {
                        var storedData = JSON.parse(FoxyDeal.extension.storage.getStoredData());
                        var keyInt;
                        for (keyInt in keysArray) {
                            if (storedData[keysArray[keyInt]] !== undefined) {
                                delete storedData[keysArray[keyInt]];
                            }
                        }
                        FoxyDeal.extension.storage.setStoredData(JSON.stringify(storedData));
                        onRemoved();
                    } catch (ex) {

                    }
                }
            };
        })(),
        rules: {
            domains: {
                oArray: null,
                load: (function () {
                    try {
                        // object which will contain stored variables (date string and timestamp) so they don't need to be reloaded from storage every time
                        var stored = null;
                        /*
                        * check if domain array needs to be downloaded or updated
                        * if download/update is not required, execute callback and prevent further execution
                        */
                        var update = function (onLoaded) {

                            // prevent further execution if update is not required and execute callback if domain array is not empty
                            if (FoxyDeal.rules.time.string() === stored.dateString || !FoxyDeal.rules.time.past4am()) {
                                if (FoxyDeal.rules.domains.oArray.isEmpty()) {
                                    FoxyDeal.extension.log.info('Request for domain array has been sent today but it was unsuccessful.\nNew request will be sent tomorrow.');
                                } else {
                                    FoxyDeal.extension.log.info('Domain array found in storage.\nRequest for updates doesn\'t need to be sent until tomorrow.');
                                    if (onLoaded) {
                                        onLoaded();
                                    }
                                }
                                stored.dateString = stored.dateString || FoxyDeal.rules.time.string();
                                return;
                            }

                            stored.dateString = stored.dateString || FoxyDeal.rules.time.string();

                            // build url
                            var lastUpdate = stored.lastUpdate;
                            var url = 'rules/shops';
                            if (lastUpdate) {
                                url += '/updated/' + lastUpdate;
                                FoxyDeal.extension.log.info('Domain array needs to be updated.');
                            } else {
                                FoxyDeal.extension.log.info('Domain array not found in storage.');
                            }

                            // store domain array, timestamp and current dateString so that next request is not sent until next day, regardless of the outcome of this request
                            FoxyDeal.rules.domains.save(lastUpdate);

                            // download object with domains and timestamp
                            FoxyDeal.rules.download(url, function (domainsObj, httpStatus) {
                                try {
                                    if (httpStatus === 200) {
                                        var newDomains = [];
                                        // check if server response was valid
                                        if (domainsObj) {
                                            if (domainsObj.s) {
                                                newDomains = domainsObj.s;
                                            }
                                            if (domainsObj.t) {
                                                lastUpdate = domainsObj.t;
                                            }
                                        }

                                        for (var i = 0; i < newDomains.length; i++) {
                                            if (!FoxyDeal.rules.domains.oArray.hasItem(newDomains[i])) {
                                                // add new domain to domain array
                                                FoxyDeal.rules.domains.oArray.addItem(newDomains[i]);
                                            } else {
                                                /*
                                                * domain is already in array which means that rules have been updated
                                                * delete rules from storage in order to receive new rules when/if required
                                                */
                                                if (FoxyDeal.rules.shopRules.container[newDomains[i]]) {
                                                    delete FoxyDeal.rules.shopRules.container[newDomains[i]];
                                                }
                                            }
                                        }

                                    } else {
                                        // got  invalid response from API
                                        FoxyDeal.extension.log.info('Received invalid response from server, HTTP status code:' + httpStatus);
                                    }
                                } catch (ex) {

                                }

                                // store domain array, timestamp and current dateString
                                FoxyDeal.rules.domains.save(lastUpdate);

                                // store shop rules
                                FoxyDeal.storage.set({
                                    rulesContainer: FoxyDeal.rules.shopRules.container
                                });

                                // execute callback if domain array is not empty
                                if (!FoxyDeal.rules.domains.oArray.isEmpty() && onLoaded) {
                                    onLoaded();
                                }

                                FoxyDeal.extension.log.info('Got response from API.\nhttp status code: ' + httpStatus + '\nresponse body: ' + JSON.stringify(domainsObj));
                            });
                        };
                    } catch (ex) {
                        FoxyDeal.extension.log.error(ex.message);
                    }

                    return function (onLoaded) {
                        /*
                        * loads domain array from storage, if previously stored (and updates it if required)
                        * else, downloads domain array
                        */
                        if (!stored || (FoxyDeal.rules.time.string() !== stored.dateString && FoxyDeal.rules.time.past4am())) {
                            FoxyDeal.storage.get({
                                domainsContainer: {
                                    domainArray: [],
                                    dateString: '',
                                    lastUpdate: null
                                }
                            }, function (storedObj) {
                                FoxyDeal.rules.domains.oArray = new FoxyDeal.oArray(storedObj.domainsContainer.domainArray);

                                stored = {
                                    dateString: storedObj.domainsContainer.dateString,
                                    lastUpdate: storedObj.domainsContainer.lastUpdate
                                };

                                // execute updater
                                try {
                                    update(onLoaded);
                                } catch (ex) {
                                    FoxyDeal.extension.log.error(ex.message);
                                }
                            });
                        }
                    }
                })(),
                save: function (lastUpdate) {
                    // store domain array, timestamp and current dateString
                    FoxyDeal.storage.set({
                        domainsContainer: {
                            domainArray: FoxyDeal.rules.domains.oArray.toArray(),
                            dateString: FoxyDeal.rules.time.string(),
                            lastUpdate: lastUpdate
                        }
                    });
                }
            },
            shopRules: {
                container: {},
                load: function (onLoaded) {
                    FoxyDeal.storage.get({
                        rulesContainer: {}
                    }, function (storedObj) {
                        FoxyDeal.rules.shopRules.container = storedObj.rulesContainer;
                        onLoaded();
                    });
                },
                get: function (domain, onLoaded) {
                    /*
                    * loads shop rules from storage or downloads them if they are not stored
                    */

                    try {
                        // query 'container' for shop rules
                        var rules = FoxyDeal.rules.shopRules.container[domain];

                        // construct/fix rules object if required
                        if (!rules) {
                            rules = {};
                        }
                        if (!rules.previousRequest) {
                            rules.previousRequest = 0;
                        }
                        if (!rules.pageRules) {
                            rules.pageRules = null;
                        }

                        // if rules were previously stored return them via callback and prevent further execution
                        if (rules.pageRules) {
                            onLoaded(rules.pageRules);
                            return;
                        }

                        // prevent further execution if less than 24h have passed since last request
                        if (!FoxyDeal.rules.time.dayPassedSince(rules.previousRequest)) {
                            FoxyDeal.extension.log.info('Previous rules request for "' + domain + '" has failed.\nIt\'s been less than 24h since previous attempt so new request won\'t be sent now.');
                            return;
                        }

                        // save rules in order to prevent additional request in next 24h, regardless of the outcome of this request
                        FoxyDeal.rules.shopRules.save(domain, null);

                        // build URL
                        var url = 'rules/shop/' + domain;

                        FoxyDeal.extension.log.info('Rules for "' + domain + '" need to be downloaded.');

                        // download rules
                        FoxyDeal.rules.download(url, function (pageRules, httpStatus) {
                            if (httpStatus === 200) {
                                // save rules
                                FoxyDeal.rules.shopRules.save(domain, pageRules);

                                // return rules via callback
                                onLoaded(pageRules);
                            } else if (httpStatus === 410) {
                                // delete shop
                                FoxyDeal.rules.deleteShop(domain);
                            }
                        });
                    } catch (exc) {
                        FoxyDeal.extension.log.error(exc.message);
                    }

                },
                save: function (domain, pageRules) {
                    // add/overwrite rules object in container
                    FoxyDeal.rules.shopRules.container[domain] = {
                        pageRules: pageRules,
                        previousRequest: FoxyDeal.rules.time.stamp()
                    };

                    // store container
                    FoxyDeal.storage.set({
                        rulesContainer: FoxyDeal.rules.shopRules.container
                    });
                }
            },
            time: {
                stamp: function (ms) {
                    /*
                    * returns timestamp
                    * @ms - optional, if set to 'true' timestamp in milliseconds is returned
                    * else, timestamp in seconds is returned
                    */
                    if (ms === true) {
                        var d = new Date();
                        return (d.getTime() + (d.getTimezoneOffset() * 60000));
                    }

                    var d = new Date();
                    return Math.round(d.getTime() / 1000) + (d.getTimezoneOffset() * 60);
                },
                string: function () {
                    return new Date().toDateString();
                },
                dayPassedSince: function (utc_timestamp) {
                    var secondsPassed = FoxyDeal.rules.time.stamp() - utc_timestamp;
                    if (secondsPassed > 86400) {
                        return true;
                    }
                    return false;
                },
                past4am: function () {
                    var d = new Date();
                    var h = d.getHours();
                    return (h - 4) > 0;
                }
            },
            download: function (relativePath, onDownloaded) {
                /*
                * intended for domain array and rules download using POST request (UID is submitted)
                * @relativePath - relative rules or domain array path (path after '/rules/')
                * @onDownloaded - callback which needs to be called when rules are downloaded and parsed
                */

                var url = FoxyDeal.api.baseURL + relativePath;
                FoxyDeal.extension.log.info('Sending request to: ' + url);
                FoxyDeal.extension.ajax(url, function (xhr) {
                    var js_obj = null;
                    // if HTTP status code is 200 try to parses JSON
                    if (xhr.status === 200) {
                        try {
                            js_obj = JSON.parse(xhr.responseText);
                        } catch (ex) {
                            FoxyDeal.extension.log.error('Exception occurred while attempting to parse data received from API.\nRecieved data:\n' + (xhr.responseText ? xhr.responseText : '') + '\nException info:' + ex.message);
                        }
                    }
                    // return resulting JS object and HTTP status code
                    onDownloaded(js_obj, xhr.status);
                }, {
                    postJSobj: {
                        uid: FoxyDeal.uid.value,
                        version: FoxyDeal.extension.version
                    }
                });
            },
            deleteShop: function (domain) {
                /* 
                * removes domain from domain array / storage and rules for this domain
                */

                // remove domain
                FoxyDeal.rules.domains.oArray.removeItem(domain);

                // remove rules
                if (FoxyDeal.rules.shopRules.container[domain]) {
                    delete FoxyDeal.rules.shopRules.container[domain];
                }

                // save changes
                FoxyDeal.storage.set({
                    domainArray: FoxyDeal.rules.domains.oArray.toArray(),
                    rulesContainer: FoxyDeal.rules.shopRules.container
                });
            }
        },
        getDomain: function (hostname) {
            /* checks if domain is in domain array
            * returns domain if it's found, else returns nullhw.FoxyDeal.SQLiteStorage
            * @hostname hostname of the page which fired "onNavigate" event
            */

            // check if domain array needs to be updated
            FoxyDeal.rules.domains.load();

            var supported_shop = false;
            var domain_parts = hostname.split('.');
            while (domain_parts.length > 1 && !supported_shop) {
                supported_shop = FoxyDeal.rules.domains.oArray.hasItem(domain_parts.join('.'));
                if (!supported_shop) {
                    domain_parts.splice(0, 1);
                } else {
                    return domain_parts.join('.');
                }
            }
            return null;
        },
        oArray: function (iArray) {
            var cObj = {};

            for (var i = 0; i < iArray.length; i++) {
                if (!cObj[iArray[i]]) {
                    cObj[iArray[i]] = true;
                }
            }

            return {
                toArray: function () {
                    return Object.keys(cObj);
                },
                isEmpty: function () {
                    for (var key in cObj) {
                        if (Object.prototype.hasOwnProperty.call(cObj, key)) return false;
                    }
                    return true;
                },
                hasItem: function (item) {
                    return cObj[item] === true ? true : false;
                },
                addItem: function (item) {
                    if (cObj[item] !== true) {
                        cObj[item] = true;
                    }
                },
                removeItem: function (item) {
                    if (cObj[item]) {
                        delete cObj[item];
                    }
                }
            };
        }
    };

    FoxyDeal.rules.domains.oArray = new FoxyDeal.oArray([]);

    FoxyDeal.extension = {
        version: '6.2.0',
        browser: 'FF',
        storage: (function () {
            FoxyDeal.hw = Components.classes["@mozilla.org/appshell/appShellService;1"].getService(Components.interfaces.nsIAppShellService).hiddenDOMWindow;
            FoxyDeal.hw.FoxyDeal = FoxyDeal.hw.FoxyDeal || {};
            if (FoxyDeal.hw.FoxyDeal.running) {
                var initFunc = FoxyDeal.initialize;
                FoxyDeal.initialize = function () { };
                FoxyDeal.hw.document.addEventListener("InitFoxyDeal", initFunc, false, true);
            } else {
                FoxyDeal.hw.FoxyDeal.running = true;
            }

            if (FoxyDeal.hw.FoxyDeal && FoxyDeal.hw.FoxyDeal.extensionStorage) {
                return FoxyDeal.hw.FoxyDeal.extensionStorage;
            }
            try {
                FoxyDeal.hw.FoxyDeal.SQLiteStorage = FoxyDeal.hw.FoxyDeal.SQLiteStorage || {
                    // DB file name
                    dbName: "foxydeal.sqlite",
                    // Pointer to the database
                    dbPointer: null,
                    // Mozilla Storage Service pointer
                    StorageService: null,
                    // Database Connection
                    dbConn: null,
                    initialize: function () {
                        // open DB connection
                        FoxyDeal.hw.FoxyDeal.SQLiteStorage.dbPointer = Cc["@mozilla.org/file/directory_service;1"].getService(Ci.nsIProperties).get("ProfD", Ci.nsIFile);
                        FoxyDeal.hw.FoxyDeal.SQLiteStorage.dbPointer.append(FoxyDeal.hw.FoxyDeal.SQLiteStorage.dbName);
                        FoxyDeal.hw.FoxyDeal.SQLiteStorage.StorageService = Cc["@mozilla.org/storage/service;1"].getService(Ci.mozIStorageService);
                        FoxyDeal.hw.FoxyDeal.SQLiteStorage.dbConn = FoxyDeal.hw.FoxyDeal.SQLiteStorage.StorageService.openDatabase(FoxyDeal.hw.FoxyDeal.SQLiteStorage.dbPointer);

                        // set-up DB
                        FoxyDeal.hw.FoxyDeal.SQLiteStorage.setUp();
                    },
                    createTable: function () {
                        FoxyDeal.hw.FoxyDeal.SQLiteStorage.dbConn.executeSimpleSQL('CREATE TABLE localStorage (JSON TEXT)');
                        FoxyDeal.hw.FoxyDeal.SQLiteStorage.dbConn.executeSimpleSQL('INSERT INTO localStorage (JSON) VALUES (\'{}\')');
                    },
                    setUp: function () {
                        // create table if it doesn't exist
                        if (!FoxyDeal.hw.FoxyDeal.SQLiteStorage.dbConn.tableExists('localStorage')) {
                            FoxyDeal.hw.FoxyDeal.SQLiteStorage.createTable();
                            return;
                        }

                        // check if this table has column 'JSON'
                        var szStatement = 'PRAGMA table_info(localStorage)';
                        var dbStatement = FoxyDeal.hw.FoxyDeal.SQLiteStorage.dbConn.createStatement(szStatement);
                        while (dbStatement.executeStep()) {
                            if (dbStatement.getString(1) == 'JSON') {
                                dbStatement.reset();
                                return;
                            }
                        }
                        dbStatement.reset();

                        // if table doesn't have column 'JSON', delete the table and create it again
                        FoxyDeal.hw.FoxyDeal.SQLiteStorage.dbConn.executeSimpleSQL('DROP TABLE localStorage');
                        FoxyDeal.hw.FoxyDeal.SQLiteStorage.createTable();
                    }

                };

                FoxyDeal.hw.FoxyDeal.SQLiteStorage.initialize();

                var storedDataContainer = null;

                FoxyDeal.hw.FoxyDeal.extensionStorage = {
                    getStoredData: function () {
                        if (storedDataContainer === null) {
                            // gets data from DB
                            var szStatement = 'select JSON from localStorage';
                            var dbStatement = FoxyDeal.hw.FoxyDeal.SQLiteStorage.dbConn.createStatement(szStatement);

                            dbStatement.executeStep();

                            storedDataContainer = dbStatement.getString(0);
                        }
                        return storedDataContainer;
                    },
                    setStoredData: function (newValue) {
                        /* updates data in DB
                        * @newValue new value
                        */
                        try {
                            storedDataContainer = newValue;
                            var dbStatement = FoxyDeal.hw.FoxyDeal.SQLiteStorage.dbConn.createStatement('update localStorage set JSON = :newValue');
                            dbStatement.params.newValue = newValue;
                            dbStatement.execute();
                        } catch (err) {

                        }
                    }
                };
                return FoxyDeal.hw.FoxyDeal.extensionStorage;
            } catch (ex) {
                return null;
            }
        })(),
        ajax: function (url, onResponse, settings) {
            /*
            * downloads data from web resource and returns it via callback along with HTTP status code
            * @url - url of web resource
            * @onResponse - callback which is called when response is received
            * @settings - optional, object which contains additional parameter(s)
            * @settings.headers - object with form: {header_name:header_value} (any number of headers can be used)
            * @settings.postStr - string which needs to be sent via POST request
            * @settings.postJSobj - JS object which needs to be sent via POST request (object is stringified before sending so JSON is sent)
            */
            try {
                if (!url || !onResponse) {
                    return;
                }

                if (!settings) {
                    settings = {};
                }

                if (!settings.headers) {
                    settings.headers = {};
                }

                if (!settings.postStr) {
                    if (settings.postJSobj) {
                        settings.postStr = JSON.stringify(settings.postJSobj);
                    } else {
                        settings.postStr = null;
                    }
                }

                var method = settings.postStr ? 'POST' : 'GET';

                var xhr = new XMLHttpRequest();
                xhr.open(method, url, true);
                xhr.overrideMimeType('text/plain');

                xhr.setRequestHeader('Content-type', 'application/json');
                var header;
                for (header in settings.headers) {
                    xhr.setRequestHeader(header, settings.headers[header]);
                }
                xhr.onreadystatechange = function () {
                    if (xhr.readyState == 4 /* complete */) {
                        onResponse(xhr);
                    }
                }
                xhr.send(settings.postStr);
            } catch (ex) {
                FoxyDeal.extension.log.error('Exception occurred while sending HTTP request to: ' + url + '\nException info: ' + ex.message);
            }
        },
        injectJS: (function () {
            var injectInfo = function (doc, info) {
                info.browser = { FF: true };
                try {
                    var scriptEl = doc.createElement('script');
                    scriptEl.appendChild(doc.createTextNode('window.FoxyDealInfo = ' + JSON.stringify(info) + ';'));
                    doc.getElementsByTagName("head")[0].appendChild(scriptEl);
                } catch (ex) {
                    FoxyDeal.extension.log.error(ex.message);
                }
            };

            var injectMainJs = function (doc) {
                try {
                    var scriptEl = doc.createElement('script');
                    scriptEl.setAttribute('src', FoxyDeal.cdn.baseURL + 'main.js');
                    doc.getElementsByTagName("head")[0].appendChild(scriptEl);
                } catch (ex) {
                    FoxyDeal.extension.log.error(ex.message);
                }
            };

            return function (doc, info) {
                FoxyDeal.extension.log.info('Injecting rules, partner info, uid and main.js, url: ' + doc.location.href);
                injectInfo(doc, info);
                injectMainJs(doc);
            }
        })(),
        navigateListener: (function () {
            var navigateListener = {
                QueryInterface: function (aIID) {
                    if (aIID.equals(Components.interfaces.nsIWebProgressListener) || aIID.equals(Components.interfaces.nsISupportsWeakReference) || aIID.equals(Components.interfaces.nsISupports)) {
                        return this;
                    }
                    throw Components.results.NS_NOINTERFACE;
                },
                onLocationChange: function (aBrowser, aWebProgress, aRequest, aLocation, aFlags) {
                    try {
                        aWebProgress.addProgressListener(progressListener, aWebProgress.NOTIFY_PROGRESS);
                    } catch (exc) {

                    }
                }
            };

            var progressListener = {
                QueryInterface: function (aIID) {
                    if (aIID.equals(Components.interfaces.nsIWebProgressListener) || aIID.equals(Components.interfaces.nsISupportsWeakReference) || aIID.equals(Components.interfaces.nsISupports)) {
                        return this;
                    }
                    throw Components.results.NS_NOINTERFACE;
                },
                onProgressChange: function (aWebProgress, aRequest, curSelf, maxSelf, curTot, maxTot) {
                    var doc = aWebProgress.DOMWindow.document;
                    if (doc.body) {
                        aWebProgress.removeProgressListener(this);
                        var doc = aWebProgress.DOMWindow.document;
                        FoxyDeal.onNavigate(doc.location.hostname, doc.location.href, doc);
                    } else if (!aWebProgress.isLoadingDocument) {
                        aWebProgress.removeProgressListener(this);
                    }
                }
            };

            var registered = false;

            return {
                register: function () {
                    /*
                    * registers 'onNavigate' listener if it hasn't been registered already
                    */
                    if (registered) {
                        return;
                    }
                    try {
                        gBrowser.addTabsProgressListener(navigateListener);
                        registered = true;
                    } catch (ex) {

                    }
                    if (FoxyDeal.hw.FoxyDeal.running) {
                        FoxyDeal.hw.FoxyDeal.running = false;
                        var ev = FoxyDeal.hw.document.createEvent("Events");
                        ev.initEvent("InitFoxyDeal", true, false);
                        FoxyDeal.hw.document.dispatchEvent(ev);
                    }
                },
                unregister: function () {
                    /*
                    * unregisters 'onNavigate' listener if it is currently registered
                    */
                    if (!registered) {
                        return;
                    }
                    try {
                        gBrowser.removeTabsProgressListener(navigateListener);
                        registered = false;
                    } catch (ex) {

                    }
                }
            }
        })(),
        cleanup: function () {
            // removes event listeners, etc so that extension can be disabled
            try {
                FoxyDeal.extension.navigateListener.unregister();
            } catch (exc) {

            }
        },
        log: (function () {
            // var consoleService = Components.classes["@mozilla.org/consoleservice;1"].getService(Components.interfaces.nsIConsoleService);
            return {
                error: function (log_msg) {
                    // consoleService.logStringMessage("[FoxyDeal extension] ERROR: " + log_msg);
                },
                info: function (log_msg) {
                    // consoleService.logStringMessage("[FoxyDeal extension] INFO: " + log_msg);
                }
            }
        })()
    };

    FoxyDeal.storage.get({
        storageVersion: '1.0'
    }, function (storedObj) {
        if (storedObj.storageVersion === '1.0') {
            FoxyDeal.storage.remove(['uidRequests', 'domainArray', 'dateString', 'lastUpdate'], function () {
                FoxyDeal.storage.set({ storageVersion: '2.0' });
            });
        }
    });

    FoxyDeal.isEnabled = null;
    FoxyDeal.enable = (function () {
        var initialized = false;
        return function () {
            /*
            * enables FoxyDeal
            * FoxyDeal will remain enabled even after browser is restarted
            * to disable foxydeal 'FoxyDeal.disable()' needs to be called
            */
            if (!FoxyDeal.extension.storage || FoxyDeal.isEnabled) {
                return;
            }

            if (!initialized) {
                initialized = true;
                FoxyDeal.storage.set({ enabled: true }, function () {
                    FoxyDeal.initialize();
                });
            }

            FoxyDeal.isEnabled = true;
            for (var i = 0; i < FoxyDeal.enabledChangeCallbacks.length; i++) {
                FoxyDeal.enabledChangeCallbacks[i](true);
            }
        }
    })();
    FoxyDeal.disable = function () {
        /*
        * disables FoxyDeal
        * FoxyDeal will remain disabled even after browser is restarted
        * to enable FoxyDeal 'FoxyDeal.enable()' needs to be called
        */
        if (!FoxyDeal.extension.storage || FoxyDeal.isEnabled === false) {
            return;
        }
        FoxyDeal.isEnabled = false;
        FoxyDeal.storage.set({ enabled: false });
        for (var i = 0; i < FoxyDeal.enabledChangeCallbacks.length; i++) {
            FoxyDeal.enabledChangeCallbacks[i](false);
        }
    };
    FoxyDeal.enabledChangeCallbacks = [];
    FoxyDeal.onEnabledChange = function (callback) {
        FoxyDeal.enabledChangeCallbacks.push(callback);
    };

    FoxyDeal.run = function (options) {
        if (!FoxyDeal.storage) {
            return;
        }

        var onLoad = function () {
            FoxyDeal.storage.get({ enabled: null }, function (storedObj) {
                FoxyDeal.isEnabled = storedObj.enabled;

                var majorShops = ["amazon.com", "amazon.de", "amazon.fr", "amazon.co.uk", "amazon.it", "amazon.ca", "amazon.at", "amazon.es", "ebay.com", "ebay.co.uk", "ebay.com.au", "ebay.fr", "ebay.de"];

                var message = "Additional feature from Proxtube: Enable price-comparisons and coupon alerts while you shop?";
                var buttons = [{
                    label: 'Yes',
                    accessKey: 'Y',
                    callback: function () {
                        FoxyDeal.enable();
                        notifyBox.removeCurrentNotification();
                    }
                }, {
                    label: 'No',
                    accessKey: 'N',
                    callback: function () {
                        notifyBox.removeCurrentNotification();
                    }
                }, {
                    label: 'No and don\'t ask me again.',
                    accessKey: 'D',
                    callback: function () {
                        FoxyDeal.disable();
                        notifyBox.removeCurrentNotification();
                    }
                }, {
                    label: "Learn more...",
                    accessKey: "M",
                    callback: function () {
                        gBrowser.selectedTab = gBrowser.addTab('https://proxtube.com/foxydeal.php');
                    }
                }];

                var optInPages = ['^https?:\/\/s3\.amazonaws\.com\/(www\.|staging\.cdn\.|dev\.cdn\.|cdn\.)?foxydeal\.com\/', '^https?:\/\/[^\/]*foxydeal\.com\/'];

                var onDCL = function (evt) {
                    var doc = evt.originalTarget;

                    var majorShop = (function () {
                        var supported_shop = false;
                        var hostname = doc.location.hostname;
                        var domain_parts = hostname.split('.');
                        while (domain_parts.length > 1 && !supported_shop) {
                            supported_shop = majorShops.indexOf(domain_parts.join('.')) > -1;
                            if (!supported_shop) {
                                domain_parts.splice(0, 1);
                            } else {
                                return true;
                            }
                        }
                        return false;
                    })();

                    var optinPage = false;
                    for (var i = 0; i < optInPages.length; i++) {
                        var rx = new RegExp(optInPages[i], "i");
                        if (rx.test(doc.location.href)) {
                            optinPage = true;
                            break;
                        }
                    }

                    var displayHide = function (isEnabled) {
                        var fd_show_if_disabled = doc.getElementById('fd-show-if-disabled');
                        var fd_show_if_enabled = doc.getElementById('fd-show-if-enabled');

                        if (fd_show_if_enabled) {
                            fd_show_if_enabled.style.display = isEnabled ? "block" : "none";
                        }
                        if (fd_show_if_disabled) {
                            fd_show_if_disabled.style.display = isEnabled ? "none" : "block";
                        }
                    };

                    if (majorShop) {
                        if (FoxyDeal.isEnabled !== null) {
                            return;
                        }

                        var notifyBox = gBrowser.getNotificationBox();
                        notifyBox.appendNotification(message, "foxydeal-popup", null, notifyBox.PRIORITY_INFO_MEDIUM, buttons);
                    } else if (optinPage) {
                        displayHide(FoxyDeal.isEnabled);
                        var fd_enable = doc.getElementById('fd-enable');
                        if (fd_enable) {
                            // register click listener
                            fd_enable.addEventListener('click', function () {
                                FoxyDeal.enable();
                                displayHide(FoxyDeal.isEnabled);
                            }, false);
                        }

                        var fd_disable = doc.getElementById('fd-disable');
                        if (fd_disable) {
                            // register click listener
                            fd_disable.addEventListener('click', function () {
                                FoxyDeal.disable();
                                displayHide(FoxyDeal.isEnabled);
                            }, false);
                        }
                    }
                };

                gBrowser.addEventListener("DOMContentLoaded", onDCL, false);

                FoxyDeal.optin = {
                    addPage: function (url) {
                        optInPages.push(url);
                    }
                };

                FoxyDeal.notification = {
                    openPage: function (url, extensionID) {
                        if (!url) {
                            return;
                        }

                        var getCurrentVersion = function (extensionID, onDone) {
                            try {
                                Components.utils.import('resource://gre/modules/AddonManager.jsm');
                                AddonManager.getAddonByID(extensionID, function (addon) {
                                    onDone(addon.version);
                                });
                            } catch (ex) {
                                try {
                                    var em = Components.classes['@mozilla.org/extensions/manager;1'].getService(Components.interfaces.nsIExtensionManager);
                                    var addon = em.getItemForID(extensionID);
                                    onDone(addon.version);
                                } catch (exc) {
                                    onDone(null);
                                }
                            }
                        };

                        var showNotification = function (version) {
                            FoxyDeal.storage.set({ notificationShown: version });
                            setTimeout(function () {
                                gBrowser.selectedTab = gBrowser.addTab(url);
                            }, 1500);
                        };

                        FoxyDeal.storage.get({ notificationShown: false, enabled: false }, function (storedObj) {
                            if ((storedObj.notificationShown && !extensionID)) {
                                return;
                            }
                            if (extensionID) {
                                getCurrentVersion(extensionID, function (version) {
                                    if (storedObj.notificationShown === version) {
                                        return;
                                    }
                                    if (storedObj.enabled) {
                                        FoxyDeal.storage.set({ notificationShown: version });
                                        return;
                                    }
                                    showNotification(version);
                                });
                            } else {
                                showNotification(true);
                            }
                        });

                    }
                };

                FoxyDeal.cleanup = function () {
                    // removes event listeners, etc so that extension can be disabled
                    try {
                        FoxyDeal.extension.navigateListener.unregister();
                        gBrowser.removeEventListener("DOMContentLoaded", onDCL, false);
                    } catch (exc) {

                    }
                };

                if (options.partner) {
                    FoxyDeal.partner = options.partner;
                }

                if (options.notification && options.notification.page) {
                    var extensionID = options.notification.showOnEveryUpdate ? options.notification.showOnEveryUpdate.extensionID : null;
                    extensionID = extensionID ? extensionID.toString() : null;
                    FoxyDeal.notification.openPage(options.notification.page, extensionID);
                }

                if (options.optinPages instanceof Array) {
                    optInPages = optInPages.concat(options.optinPages);
                }

                if (options.beforeInit) {
                    options.beforeInit();
                }

                if (FoxyDeal.isEnabled) {
                    FoxyDeal.initialize();
                }
            });
        };

        if (document.readyState == 'complete' || document.readySate == 'loaded' || document.readyState == 'interactive') {
            onLoad();
        } else {
            window.addEventListener("load", onLoad, false);
        }
    };

    FoxyDeal.run({
        partner: {
            id: 1002,
            sub_id: 'ptv6ff'
        },
        notification: {
            page: 'https://proxtube.com/foxydeal.php',
            showOnEveryUpdate: { extensionID: 'ich@maltegoetz.de' }
        },
        optinPages: ['https?:\/\/[^\/]*proxtube.com\/foxydeal\.php'],
        beforeInit: function () {
            var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);

            var FoxyDealEnabled = function () {
                try {
                    var prefsBranch = prefs.getBranch("extensions.proxtube.");
                    FoxyDeal.onEnabledChange(function (isEnabled) {
                        prefsBranch.setBoolPref("boolpref_showfoxydeal", isEnabled ? true : false);
                    });
                    return prefsBranch.getBoolPref("boolpref_showfoxydeal");
                } catch (ex) {

                }
                return false;
            };

            if (FoxyDealEnabled()) {
                FoxyDeal.enable();
            }

            var globalBranch = prefs.getBranch("");
            if (!("addObserver" in globalBranch)) {
                globalBranch.QueryInterface(Components.interfaces.nsIPrefBranch2);
            }

            globalBranch.addObserver("extensions.proxtube.boolpref_showfoxydeal", function (aSubject, aTopic, aData) {
                if (aData === "extensions.proxtube.boolpref_showfoxydeal") {
                    if (FoxyDealEnabled()) {
                        FoxyDeal.enable();
                    } else {
                        FoxyDeal.disable();
                    }
                }
            }, false);
        }
    });

}